@@ -1,8 +1,7 @@
-
 /*
  * mdadm - manage Linux "md" devices aka RAID arrays.
  *
- * Copyright (C) 2001-2006 Neil Brown <neilb@cse.unsw.edu.au>
+ * Copyright (C) 2001-2006 Neil Brown <neilb@suse.de>
  *
  *
  *    This program is free software; you can redistribute it and/or modify
@@ -30,7 +29,7 @@
 
 #include "mdadm.h"
 
-char Version[] = Name " - v2.4.1 -  4 April 2006\n";
+char Version[] = Name " - v2.5.1 -  16 June 2006\n";
 
 /*
  * File: ReadMe.c
@@ -131,13 +130,15 @@ struct option long_options[] = {
     {"spare-devices",1,0, 'x'},
     {"size",	  1, 0, 'z'},
     {"auto",	  1, 0, 'a'}, /* also for --assemble */
-    {"assume-clean",0,0, 3 },
+    {"assume-clean",0,0, AssumeClean },
     {"metadata",  1, 0, 'e'}, /* superblock format */
     {"bitmap",	  1, 0, 'b'},
-    {"bitmap-chunk", 1, 0, 4},
-    {"write-behind", 2, 0, 5},
+    {"bitmap-chunk", 1, 0, BitmapChunk},
+    {"write-behind", 2, 0, WriteBehind},
     {"write-mostly",0, 0, 'W'},
-    {"re-add",    0, 0,  6},
+    {"re-add",    0, 0,  ReAdd},
+    {"homehost",  1, 0,  HomeHost},
+    {"auto-update-homehost", 0, 0, AutoHomeHost},
 
     /* For assemble */
     {"uuid",      1, 0, 'u'},
@@ -157,10 +158,11 @@ struct option long_options[] = {
     {"stop",      0, 0, 'S'},
     {"readonly",  0, 0, 'o'},
     {"readwrite", 0, 0, 'w'},
+    {"no-degraded",0,0,  NoDegraded },
 
     /* For Detail/Examine */
     {"brief",	  0, 0, 'b'},
-    {"sparc2.2",  0, 0, 22},
+    {"sparc2.2",  0, 0, Sparc22},
     {"test",      0, 0, 't'},
 
     /* For Follow/monitor */
@@ -173,7 +175,7 @@ struct option long_options[] = {
     {"oneshot",   0, 0, '1'},
     {"pid-file",  1, 0, 'i'},
     /* For Grow */
-    {"backup-file", 1,0, 7},
+    {"backup-file", 1,0, BackupFile},
     
     {0, 0, 0, 0}
 };
@@ -271,6 +273,7 @@ char OptionHelp[] =
 "  --force       -f   : Assemble the array even if some superblocks appear out-of-date\n"
 "  --update=     -U   : Update superblock: one of sparc2.2, super-minor or summaries\n"
 "  --auto(=p)    -a   : Automatically allocate new (partitioned) md array if needed.\n"
+"  --no-degraded      : Do not start any degraded arrays - default unless --scan.\n"
 "\n"
 " For detail or examine:\n"
 "  --brief       -b   : Just print device name and UUID\n"
@@ -389,6 +392,14 @@ char Help_assemble[] =
 " Those devices are md devices that are to be assembled.  Their identity\n"
 " and components are determined from the config file.\n"
 "\n"
+" If mdadm can not find all of the components for an array, it will assemble\n"
+" it but not activate it unless --run or --scan is given.  To preserve this\n"
+" behaviour even with --scan, add --no-degraded.  Note that \"all of the\n"
+" components\" means as many as were present the last time the array was running\n"
+" as recorded in the superblock.  If the array was already degraded, and\n"
+" the missing device is not a new problem, it will still be assembled.  It\n"
+" is only newly missing devices that cause the array not to be started.\n"
+"\n"
 "Options that are valid with --assemble (-A) are:\n"
 "  --bitmap=          : bitmap file to use wit the array\n"
 "  --uuid=       -u   : uuid of array to assemble. Devices which don't\n"
@@ -403,6 +414,7 @@ char Help_assemble[] =
 "  --force       -f   : Assemble the array even if some superblocks appear\n"
 "                     : out-of-date.  This involves modifying the superblocks.\n"
 "  --update=     -U   : Update superblock: one of sparc2.2, super-minor or summaries\n"
+"  --no-degraded      : Assemble but do not start degraded arrays.\n"
 ;
 
 char Help_manage[] =
