@@ -1,7 +1,7 @@
 /*
  * mdassemble - assemble Linux "md" devices aka RAID arrays.
  *
- * Copyright (C) 2001-2002 Neil Brown <neilb@cse.unsw.edu.au>
+ * Copyright (C) 2001-2006 Neil Brown <neilb@suse.de>
  * Copyright (C) 2003 Luca Berra <bluca@vodka.it>
  *
  *
@@ -54,7 +54,7 @@ mapping_t pers[] = {
 };
 
 #ifndef MDASSEMBLE_AUTO
-/* from mdadm.c */
+/* from mdopen.c */
 int open_mddev(char *dev, int autof/*unused */)
 {
 	int mdfd = open(dev, O_RDWR, 0);
@@ -79,7 +79,7 @@ int readonly = 0;
 int verbose = 0;
 int force = 0;
 
-int main() {
+int main(int argc, char *argv[]) {
 	mddev_ident_t array_list =  conf_get_ident(configfile, NULL);
 	if (!array_list) {
 		fprintf(stderr, Name ": No arrays found in config file\n");
@@ -97,7 +97,8 @@ int main() {
 				continue;
 			rv |= Assemble(array_list->st, array_list->devname, mdfd,
 					   array_list, configfile,
-					   NULL,
-					   readonly, runstop, NULL, verbose, force);
+					   NULL, NULL,
+					   readonly, runstop, NULL, NULL, verbose, force);
 		}
+	return rv;
 }
