@@ -1,7 +1,7 @@
 /*
  * mdadm - manage Linux "md" devices aka RAID arrays.
  *
- * Copyright (C) 2001-2004 Neil Brown <neilb@cse.unsw.edu.au>
+ * Copyright (C) 2001-2006 Neil Brown <neilb@suse.de>
  *
  *
  *    This program is free software; you can redistribute it and/or modify
@@ -27,9 +27,9 @@
  *           Australia
  */
 
+#define HAVE_STDINT_H 1
 #include "mdadm.h"
-#include <asm/byteorder.h>
-
+#include "sha1.h"
 /*
  * All handling for the 0.90.0 version superblock is in
  * this file.
@@ -81,11 +81,11 @@ void super0_swap_endian(struct mdp_superblock_s *sb)
 	sb->cp_events_hi = sb->cp_events_lo;
 	sb->cp_events_lo = t32;
 
-}	
+}
 
 #ifndef MDASSEMBLE
 
-static void examine_super0(void *sbv)
+static void examine_super0(void *sbv, char *homehost)
 {
 	mdp_super_t *sb = sbv;
 	time_t atime;
@@ -95,22 +95,72 @@ static void examine_super0(void *sbv)
 	printf("          Magic : %08x\n", sb->md_magic);
 	printf("        Version : %02d.%02d.%02d\n", sb->major_version, sb->minor_version,
 	       sb->patch_version);
-	if (sb->minor_version >= 90)
-		printf("           UUID : %08x:%08x:%08x:%08x\n", sb->set_uuid0, sb->set_uuid1,
+	if (sb->minor_version >= 90) {
+		printf("           UUID : %08x:%08x:%08x:%08x", sb->set_uuid0, sb->set_uuid1,
 		       sb->set_uuid2, sb->set_uuid3);
-	else
+		if (homehost) {
+			char buf[20];
+			void *hash = sha1_buffer(homehost,
+						 strlen(homehost),
+						 buf);
+			if (memcmp(&sb->set_uuid2, hash, 8)==0)
+				printf(" (local to host %s)", homehost);
+		}
+		printf("\n");
+	} else
 		printf("           UUID : %08x\n", sb->set_uuid0);
 
 	atime = sb->ctime;
 	printf("  Creation Time : %.24s\n", ctime(&atime));
 	c=map_num(pers, sb->level);
 	printf("     Raid Level : %s\n", c?c:"-unknown-");
-	if ((int)sb->level >= 0)
+	if ((int)sb->level >= 0) {
+		int ddsks=0;
 		printf("    Device Size : %d%s\n", sb->size, human_size((long long)sb->size<<10));
+		switch(sb->level) {
+		case 1: ddsks=1;break;
+		case 4:
+		case 5: ddsks = sb->raid_disks-1; break;
+		case 6: ddsks = sb->raid_disks-2; break;
+		case 10: ddsks = sb->raid_disks / (sb->layout&255) / ((sb->layout>>8)&255);
+		}
+		if (ddsks)
+			printf("     Array Size : %llu%s\n", (unsigned long long)ddsks * sb->size,
+			       human_size(ddsks*(long long)sb->size<<10));
+	}
 	printf("   Raid Devices : %d\n", sb->raid_disks);
 	printf("  Total Devices : %d\n", sb->nr_disks);
 	printf("Preferred Minor : %d\n", sb->md_minor);
 	printf("\n");
+	if (sb->minor_version > 90 && (sb->reshape_position+1) != 0) {
+		printf("  Reshape pos'n : %llu%s\n", (unsigned long long)sb->reshape_position/2, human_size((long long)sb->reshape_position<<9));
+		if (sb->delta_disks) {
+			printf("  Delta Devices : %d", sb->delta_disks);
+			if (sb->delta_disks)
+				printf(" (%d->%d)\n", sb->raid_disks-sb->delta_disks, sb->raid_disks);
+			else
+				printf(" (%d->%d)\n", sb->raid_disks, sb->raid_disks+sb->delta_disks);
+		}
+		if (sb->new_level != sb->level) {
+			c = map_num(pers, sb->new_level);
+			printf("      New Level : %s\n", c?c:"-unknown-");
+		}
+		if (sb->new_layout != sb->layout) {
+			if (sb->level == 5) {
+				c = map_num(r5layout, sb->new_layout);
+				printf("     New Layout : %s\n", c?c:"-unknown-");
+			}
+			if (sb->level == 10) {
+				printf("     New Layout : near=%d, %s=%d\n",
+				       sb->new_layout&255,
+				       (sb->new_layout&0x10000)?"offset":"far",
+				       (sb->new_layout>>8)&255);
+			}
+		}
+		if (sb->new_chunk != sb->chunk_size)
+			printf("  New Chunksize : %d\n", sb->new_chunk);
+		printf("\n");
+	}
 	atime = sb->utime;
 	printf("    Update Time : %.24s\n", ctime(&atime));
 	printf("          State : %s\n",
@@ -131,6 +181,12 @@ static void examine_super0(void *sbv)
 		c = map_num(r5layout, sb->layout);
 		printf("         Layout : %s\n", c?c:"-unknown-");
 	}
+	if (sb->level == 10) {
+		printf("         Layout : near=%d, %s=%d\n",
+		       sb->layout&255,
+		       (sb->layout&0x10000)?"offset":"far",
+		       (sb->layout>>8)&255);
+	}
 	switch(sb->level) {
 	case 0:
 	case 4:
@@ -140,7 +196,7 @@ static void examine_super0(void *sbv)
 	case -1:
 		printf("       Rounding : %dK\n", sb->chunk_size/1024);
 		break;
-	default: break;		
+	default: break;
 	}
 	printf("\n");
 	printf("      Number   Major   Minor   RaidDevice State\n");
@@ -162,7 +218,7 @@ static void examine_super0(void *sbv)
 		if (dp->state & (1<<MD_DISK_REMOVED)) printf(" removed");
 		if (wonly) printf(" write-mostly");
 		if (dp->state == 0) printf(" spare");
-		if ((dv=map_dev(dp->major, dp->minor)))
+		if ((dv=map_dev(dp->major, dp->minor, 0)))
 			printf("   %s", dv);
 		printf("\n");
 		if (d == -1) printf("\n");
@@ -173,9 +229,17 @@ static void brief_examine_super0(void *sbv)
 {
 	mdp_super_t *sb = sbv;
 	char *c=map_num(pers, sb->level);
+	char devname[20];
+	struct stat stb;
+
+	sprintf(devname, "/dev/md%d", sb->md_minor);
+	if (stat(devname, &stb) != 0) {
+		/* /dev/mdX doesn't exist, so use /dev/md/X */
+		sprintf(devname, "/dev/md/%d", sb->md_minor);
+	}
 
 	printf("ARRAY %s level=%s num-devices=%d UUID=",
-	       get_md_name(sb->md_minor),
+	       devname,
 	       c?c:"-unknown-", sb->raid_disks);
 	if (sb->minor_version >= 90)
 		printf("%08x:%08x:%08x:%08x", sb->set_uuid0, sb->set_uuid1,
@@ -185,7 +249,7 @@ static void brief_examine_super0(void *sbv)
 	printf("\n");
 }
 
-static void detail_super0(void *sbv)
+static void detail_super0(void *sbv, char *homehost)
 {
 	mdp_super_t *sb = sbv;
 	printf("           UUID : ");
@@ -194,6 +258,14 @@ static void detail_super0(void *sbv)
 		       sb->set_uuid2, sb->set_uuid3);
 	else
 		printf("%08x", sb->set_uuid0);
+	if (homehost) {
+		char buf[20];
+		void *hash = sha1_buffer(homehost,
+					 strlen(homehost),
+					 buf);
+		if (memcmp(&sb->set_uuid2, hash, 8)==0)
+			printf(" (local to host %s)", homehost);
+	}
 	printf("\n         Events : %d.%d\n\n", sb->events_hi, sb->events_lo);
 }
 
@@ -208,6 +280,18 @@ static void brief_detail_super0(void *sbv)
 		printf("%08x", sb->set_uuid0);
 }
 #endif
+
+static int match_home0(void *sbv, char *homehost)
+{
+	mdp_super_t *sb = sbv;
+	char buf[20];
+	char *hash = sha1_buffer(homehost,
+				 strlen(homehost),
+				 buf);
+
+	return (memcmp(&sb->set_uuid2, hash, 8)==0);
+}
+
 static void uuid_from_super0(int uuid[4], void * sbv)
 {
 	mdp_super_t *super = sbv;
@@ -223,7 +307,7 @@ static void uuid_from_super0(int uuid[4], void * sbv)
 	}
 }
 
-static void getinfo_super0(struct mdinfo *info, mddev_ident_t ident, void *sbv)
+static void getinfo_super0(struct mdinfo *info, void *sbv)
 {
 	mdp_super_t *sb = sbv;
 	int working = 0;
@@ -234,8 +318,12 @@ static void getinfo_super0(struct mdinfo *info, mddev_ident_t ident, void *sbv)
 	info->array.patch_version = sb->patch_version;
 	info->array.raid_disks = sb->raid_disks;
 	info->array.level = sb->level;
+	info->array.layout = sb->layout;
 	info->array.md_minor = sb->md_minor;
 	info->array.ctime = sb->ctime;
+	info->array.utime = sb->utime;
+	info->array.chunk_size = sb->chunk_size;
+	info->component_size = sb->size*2;
 
 	info->disk.state = sb->this_disk.state;
 	info->disk.major = sb->this_disk.major;
@@ -244,10 +332,21 @@ static void getinfo_super0(struct mdinfo *info, mddev_ident_t ident, void *sbv)
 	info->disk.number = sb->this_disk.number;
 
 	info->events = md_event(sb);
+	info->data_offset = 0;
 
 	uuid_from_super0(info->uuid, sbv);
 
-	ident->name[0] = 0;
+	if (sb->minor_version > 90 && (sb->reshape_position+1) != 0) {
+		info->reshape_active = 1;
+		info->reshape_progress = sb->reshape_position;
+		info->new_level = sb->new_level;
+		info->delta_disks = sb->delta_disks;
+		info->new_layout = sb->new_layout;
+		info->new_chunk = sb->new_chunk;
+	} else
+		info->reshape_active = 0;
+
+	sprintf(info->name, "%d", sb->md_minor);
 	/* work_disks is calculated rather than read directly */
 	for (i=0; i < MD_SB_DISKS; i++)
 		if ((sb->disks[i].state & (1<<MD_DISK_SYNC)) &&
@@ -258,8 +357,13 @@ static void getinfo_super0(struct mdinfo *info, mddev_ident_t ident, void *sbv)
 }
 
 
-static int update_super0(struct mdinfo *info, void *sbv, char *update, char *devname, int verbose)
+static int update_super0(struct mdinfo *info, void *sbv, char *update,
+			 char *devname, int verbose,
+			 int uuid_set, char *homehost)
 {
+	/* NOTE: for 'assemble' and 'force' we need to return non-zero if any change was made.
+	 * For others, the return value is ignored.
+	 */
 	int rv = 0;
 	mdp_super_t *sb = sbv;
 	if (strcmp(update, "sparc2.2")==0 ) {
@@ -310,18 +414,25 @@ static int update_super0(struct mdinfo *info, void *sbv, char *update, char *dev
 				sb->disks[i].state = 0;
 	}
 	if (strcmp(update, "force")==0) {
+		__u32 ehi = sb->events_hi, elo = sb->events_lo;
 		sb->events_hi = (info->events>>32) & 0xFFFFFFFF;
 		sb->events_lo = (info->events) & 0xFFFFFFFF;
-		if (sb->level == 5 || sb->level == 4 || sb->level == 6)
+		if (sb->events_hi != ehi ||
+		    sb->events_lo != elo)
+			rv = 1;
+		if ((sb->level == 5 || sb->level == 4 || sb->level == 6) &&
+		    (sb->state & (1 << MD_SB_CLEAN)) == 0) {
 			/* need to force clean */
 			sb->state |= (1 << MD_SB_CLEAN);
+			rv = 1;
+		}
 	}
 	if (strcmp(update, "assemble")==0) {
 		int d = info->disk.number;
 		int wonly = sb->disks[d].state & (1<<MD_DISK_WRITEMOSTLY);
-		sb->disks[d].state &= ~(1<<MD_DISK_WRITEMOSTLY);
-		if (sb->disks[d].state != info->disk.state) {
-			sb->disks[d].state = info->disk.state & wonly;
+		if ((sb->disks[d].state & ~(1<<MD_DISK_WRITEMOSTLY))
+		    != info->disk.state) {
+			sb->disks[d].state = info->disk.state | wonly;
 			rv = 1;
 		}
 	}
@@ -354,6 +465,33 @@ static int update_super0(struct mdinfo *info, void *sbv, char *update, char *dev
 		sb->state &= ~(1<<MD_SB_CLEAN);
 		sb->recovery_cp = 0;
 	}
+	if (strcmp(update, "homehost") == 0 &&
+	    homehost) {
+		uuid_set = 0;
+		update = "uuid";
+		info->uuid[0] = sb->set_uuid0;
+		info->uuid[1] = sb->set_uuid1;
+	}
+	if (strcmp(update, "uuid") == 0) {
+		if (!uuid_set && homehost) {
+			char buf[20];
+			char *hash = sha1_buffer(homehost,
+						 strlen(homehost),
+						 buf);
+			memcpy(info->uuid+2, hash, 8);
+		}
+		sb->set_uuid0 = info->uuid[0];
+		sb->set_uuid1 = info->uuid[1];
+		sb->set_uuid2 = info->uuid[2];
+		sb->set_uuid3 = info->uuid[3];
+		if (sb->state & (1<<MD_SB_BITMAP_PRESENT)) {
+			struct bitmap_super_s *bm;
+			bm = (struct bitmap_super_s*)(sb+1);
+			uuid_from_super0((int*)bm->uuid, sbv);
+		}
+	}
+	if (strcmp(update, "_reshape_progress")==0)
+		sb->reshape_position = info->reshape_progress;
 
 	sb->sb_csum = calc_sb0_csum(sb);
 	return rv;
@@ -365,9 +503,17 @@ static __u64 event_super0(void *sbv)
 	return md_event(sb);
 }
 
+/*
+ * For verion-0 superblock, the homehost is 'stored' in the
+ * uuid.  8 bytes for a hash of the host leaving 8 bytes
+ * of random material.
+ * We use the first 8 bytes (64bits) of the sha1 of the
+ * host name
+ */
 
 
-static int init_super0(struct supertype *st, void **sbp, mdu_array_info_t *info, char *ignored_name)
+static int init_super0(struct supertype *st, void **sbp, mdu_array_info_t *info,
+		       unsigned long long size, char *ignored_name, char *homehost)
 {
 	mdp_super_t *sb = malloc(MD_SB_BYTES + sizeof(bitmap_super_t));
 	int spares;
@@ -376,6 +522,7 @@ static int init_super0(struct supertype *st, void **sbp, mdu_array_info_t *info,
 
 	if (info->major_version == -1) {
 		/* zeroing the superblock */
+		*sbp = sb;
 		return 0;
 	}
 
@@ -396,6 +543,8 @@ static int init_super0(struct supertype *st, void **sbp, mdu_array_info_t *info,
 		sb->set_uuid0 = random();
 	sb->ctime = time(0);
 	sb->level = info->level;
+	if (size != info->size)
+		return 0;
 	sb->size = info->size;
 	sb->nr_disks = info->nr_disks;
 	sb->raid_disks = info->raid_disks;
@@ -408,6 +557,13 @@ static int init_super0(struct supertype *st, void **sbp, mdu_array_info_t *info,
 	}
 	if (rfd >= 0)
 		close(rfd);
+	if (homehost) {
+		char buf[20];
+		char *hash = sha1_buffer(homehost,
+					 strlen(homehost),
+					 buf);
+		memcpy(&sb->set_uuid2, hash, 8);
+	}
 
 	sb->utime = sb->ctime;
 	sb->state = info->state;
@@ -430,7 +586,7 @@ static void add_to_super0(void *sbv, mdu_disk_info_t *dinfo)
 {
 	mdp_super_t *sb = sbv;
 	mdp_disk_t *dk = &sb->disks[dinfo->number];
-	
+
 	dk->number = dinfo->number;
 	dk->major = dinfo->major;
 	dk->minor = dinfo->minor;
@@ -455,9 +611,9 @@ static int store_super0(struct supertype *st, int fd, void *sbv)
 			dsize = ((unsigned long long)size)<<9;
 	}
 
-	if (dsize < MD_RESERVED_SECTORS*2)
+	if (dsize < MD_RESERVED_SECTORS*2*512)
 		return 2;
-	
+
 	offset = MD_NEW_SIZE_SECTORS(dsize>>9);
 
 	offset *= 512;
@@ -468,6 +624,13 @@ static int store_super0(struct supertype *st, int fd, void *sbv)
 	if (write(fd, super, sizeof(*super)) != sizeof(*super))
 		return 4;
 
+	if (super->state & (1<<MD_SB_BITMAP_PRESENT)) {
+		struct bitmap_super_s * bm = (struct bitmap_super_s*)(super+1);
+		if (__le32_to_cpu(bm->magic) == BITMAP_MAGIC)
+			if (write(fd, bm, sizeof(*bm)) != sizeof(*bm))
+			    return 5;
+	}
+
 	fsync(fd);
 	return 0;
 }
@@ -515,8 +678,8 @@ static int compare_super0(void **firstp, void *secondv)
 	if (second->md_magic != MD_SB_MAGIC)
 		return 1;
 	if (!first) {
-		first = malloc(MD_SB_BYTES);
-		memcpy(first, second, MD_SB_BYTES);
+		first = malloc(MD_SB_BYTES + sizeof(struct bitmap_super_s));
+		memcpy(first, second, MD_SB_BYTES + sizeof(struct bitmap_super_s));
 		*firstp = first;
 		return 0;
 	}
@@ -551,6 +714,8 @@ static int load_super0(struct supertype *st, int fd, void **sbp, char *devname)
 	unsigned long long dsize;
 	unsigned long long offset;
 	mdp_super_t *super;
+	int uuid[4];
+	struct bitmap_super_s *bsb;
     
 #ifdef BLKGETSIZE64
 	if (ioctl(fd, BLKGETSIZE64, &dsize) != 0)
@@ -571,7 +736,7 @@ static int load_super0(struct supertype *st, int fd, void **sbp, char *devname)
 				devname, size);
 		return 1;
 	}
-	
+
 	offset = MD_NEW_SIZE_SECTORS(dsize>>9);
 
 	offset *= 512;
@@ -620,6 +785,27 @@ static int load_super0(struct supertype *st, int fd, void **sbp, char *devname)
 		st->max_devs = MD_SB_DISKS;
 	}
 
+	/* Now check on the bitmap superblock */
+	if ((super->state & (1<<MD_SB_BITMAP_PRESENT)) == 0)
+		return 0;
+	/* Read the bitmap superblock and make sure it looks
+	 * valid.  If it doesn't clear the bit.  An --assemble --force
+	 * should get that written out.
+	 */
+	if (read(fd, super+1, sizeof(struct bitmap_super_s))
+	    != sizeof(struct bitmap_super_s))
+		goto no_bitmap;
+
+	uuid_from_super0(uuid, super);
+	bsb = (struct bitmap_super_s *)(super+1);
+	if (__le32_to_cpu(bsb->magic) != BITMAP_MAGIC ||
+	    memcmp(bsb->uuid, uuid, 16) != 0)
+		goto no_bitmap;
+	return 0;
+
+ no_bitmap:
+	super->state &= ~(1<<MD_SB_BITMAP_PRESENT);
+
 	return 0;
 }
 
@@ -652,24 +838,23 @@ static __u64 avail_size0(struct supertype *st, __u64 devsize)
 	return MD_NEW_SIZE_SECTORS(devsize);
 }
 
-static int add_internal_bitmap0(struct supertype *st, void *sbv, int chunk, int delay, int write_behind, int *sizep, int may_change)
+static int add_internal_bitmap0(struct supertype *st, void *sbv, int chunk, int delay, int write_behind, unsigned long long size, int may_change, int major)
 {
 	/*
 	 * The bitmap comes immediately after the superblock and must be 60K in size
 	 * at most.  The default size is between 30K and 60K
 	 *
-	 * size is in K,  chunk is in bytes !!!
+	 * size is in sectors,  chunk is in bytes !!!
 	 */
-	unsigned long long size = *sizep;
 	unsigned long long bits;
 	unsigned long long max_bits = 60*1024*8;
 	unsigned long long min_chunk;
 	mdp_super_t *sb = sbv;
 	bitmap_super_t *bms = (bitmap_super_t*)(((char*)sb) + MD_SB_BYTES);
 
-	
+
 	min_chunk = 4096; /* sub-page chunks don't work yet.. */
-	bits = (size * 1024)/ min_chunk +1;
+	bits = (size * 512) / min_chunk + 1;
 	while (bits > max_bits) {
 		min_chunk *= 2;
 		bits = (bits+1)/2;
@@ -681,22 +866,20 @@ static int add_internal_bitmap0(struct supertype *st, void *sbv, int chunk, int
 
 	sb->state |= (1<<MD_SB_BITMAP_PRESENT);
 
-	memset(bms, sizeof(*bms), 0);
+	memset(bms, 0, sizeof(*bms));
 	bms->magic = __cpu_to_le32(BITMAP_MAGIC);
-	bms->version = __cpu_to_le32(BITMAP_MAJOR);
+	bms->version = __cpu_to_le32(major);
 	uuid_from_super0((int*)bms->uuid, sb);
 	bms->chunksize = __cpu_to_le32(chunk);
 	bms->daemon_sleep = __cpu_to_le32(delay);
-	bms->sync_size = __cpu_to_le64(size<<1);
+	bms->sync_size = __cpu_to_le64(size);
 	bms->write_behind = __cpu_to_le32(write_behind);
 
-
-
 	return 1;
 }
-		
 
-void locate_bitmap0(struct supertype *st, int fd)
+
+void locate_bitmap0(struct supertype *st, int fd, void *sbv)
 {
 	unsigned long long dsize;
 	unsigned long size;
@@ -713,7 +896,7 @@ void locate_bitmap0(struct supertype *st, int fd)
 
 	if (dsize < MD_RESERVED_SECTORS*2)
 		return;
-	
+
 	offset = MD_NEW_SIZE_SECTORS(dsize>>9);
 
 	offset *= 512;
@@ -746,8 +929,8 @@ int write_bitmap0(struct supertype *st, int fd, void *sbv)
 	}
 
 	if (dsize < MD_RESERVED_SECTORS*2)
-		return -1;
-	
+	return -1;
+
 	offset = MD_NEW_SIZE_SECTORS(dsize>>9);
 
 	offset *= 512;
@@ -756,7 +939,9 @@ int write_bitmap0(struct supertype *st, int fd, void *sbv)
 		return 3;
 
 
-	write(fd, ((char*)sb)+MD_SB_BYTES, sizeof(bitmap_super_t));
+	if (write(fd, ((char*)sb)+MD_SB_BYTES, sizeof(bitmap_super_t)) !=
+	    sizeof(bitmap_super_t))
+		return -2;
 	towrite = 64*1024 - MD_SB_BYTES - sizeof(bitmap_super_t);
 	memset(buf, 0xff, sizeof(buf));
 	while (towrite > 0) {
@@ -783,6 +968,7 @@ struct superswitch super0 = {
 	.detail_super = detail_super0,
 	.brief_detail_super = brief_detail_super0,
 #endif
+	.match_home = match_home0,
 	.uuid_from_super = uuid_from_super0,
 	.getinfo_super = getinfo_super0,
 	.update_super = update_super0,
